/** library to query measurements from Aosong DHT11 temperature and relative humidity sensor
 *  @file
 *  @author King Kévin <kingkevin@cuvoodoo.info>
 *  @copyright SPDX-License-Identifier: GPL-3.0-or-later
 *  @date 2017-2020
 *  @note peripherals used: timer channel @ref sensor_dht11_timer (add external pull-up resistor)
 */
#pragma once
#error not converted for STM32F4

/** a measurement response has been received */
extern volatile bool sensor_dht11_measurement_received;

/** measurement returned by sensor */
struct sensor_dht11_measurement_t {
	uint8_t humidity; /**< relative humidity in %RH (20-95) */
	uint8_t temperature; /**< temperature in °C (0-50) */
};

/** setup peripherals to communicate with sensor */
void sensor_dht11_setup(void);
/** request measurement from sensor
 *  @return request started successfully
 */
bool sensor_dht11_measurement_request(void);
/** decode received measurement
 *  @return decoded measurement (0xff,0xff if invalid)
 */
struct sensor_dht11_measurement_t sensor_dht11_measurement_decode(void);
